package cn.com.taiji.service.impl;

import cn.com.taiji.constants.LocationConstants;
import cn.com.taiji.domain.*;
import cn.com.taiji.domain.dto.LeaceOutDto;
import cn.com.taiji.service.FusionAnalysisService;
import cn.com.taiji.utils.DateUtils;
import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.http.client.config.RequestConfig;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.HttpAsyncResponseConsumerFactory;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.geo.GeoPoint;
import org.elasticsearch.core.TimeValue;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.text.ParseException;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.CopyOnWriteArraySet;

import static cn.com.taiji.constants.EsIndexConstants.*;


/**
 * @author xhl
 * @date 2023/2/6
 */
@Service
public class FusionAnalysisServiceImpl implements FusionAnalysisService {

    private static final RequestOptions COMMON_OPTIONS;

    static {
        RequestConfig requestConfig = RequestConfig.custom()
                .setConnectTimeout(5000)
                .setSocketTimeout(60000)
                .build();
        RequestOptions.Builder builder = RequestOptions.DEFAULT.toBuilder();
        builder.setRequestConfig(requestConfig).setHttpAsyncResponseConsumerFactory(
                // 设置查询内容大小限制,默认100 * 1024 * 1024
                new HttpAsyncResponseConsumerFactory.HeapBufferedResponseConsumerFactory(5*1024 * 1024 * 1024)
        );
        COMMON_OPTIONS = builder.build();
    }

    @Autowired
    private RestHighLevelClient client;

    @Override
    public void fusionAnalysisExcelOne(FusionQuery query, HttpServletResponse response) throws IOException, ParseException {
        List<FusionData> list = new CopyOnWriteArrayList<>();
        Date startTime = DateUtils.parse(query.getStartTime(),"yyyy-MM-dd");
        String time = DateUtils.format(startTime,"yyyy-MM-dd");
        SearchRequest request = new SearchRequest(INDEX_SEAT_FK_REALTIMETRAJECTORY+time);
        // 构建查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("mergeTime").gte(query.getStartTime()).lte(query.getEndTime()));
        double[] doubles = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
        List<GeoPoint> points = new ArrayList<>();
        for(int i = 0 ;i < doubles.length;i=i+2) {
            points.add(new GeoPoint(doubles[i+1],doubles[i]));
        }
        boolQueryBuilder.filter(QueryBuilders.geoPolygonQuery("location", points));
        searchSourceBuilder.query(boolQueryBuilder);
//        TermsAggregationBuilder aggBuilder = AggregationBuilders.terms("ship_field_agg")
//                .field("mergeTarget")
//                .size(query.getPageSize())
//                .subAggregation(
//                        AggregationBuilders.topHits("ship_field_agg_top1")
//                                .size(100)
//                                .sort(
//                                        SortBuilders.fieldSort("mergeTime").order(SortOrder.ASC)
//                                )
//                );
        searchSourceBuilder
                .trackTotalHits(true)
                .sort(SortBuilders.fieldSort("mergeTime").order(SortOrder.ASC))
//                .aggregation(aggBuilder)
                .size(query.getPageSize())
                .timeout(TimeValue.timeValueHours(1L))
                .timeout(TimeValue.timeValueMinutes(30L))
                .timeout(TimeValue.timeValueSeconds(500L));
        request.source(searchSourceBuilder);
        SearchResponse search = client.search(request, COMMON_OPTIONS);
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        List<Map<String, Object>> inList = new CopyOnWriteArrayList<Map<String, Object>>();
        List<Map<String, Object>> outList = new CopyOnWriteArrayList<Map<String, Object>>();
        for (SearchHit hit : hits) {
            Map<String, Object> map = new ConcurrentHashMap();
            map = hit.getSourceAsMap();
            inList.add(map);
        }
        outList = change(inList, "mergeTarget", outList);
        outList.forEach(map->{
            JSONObject maps = JSONObject.parseObject(JSON.toJSONString(map));
//            JSONObject maps = new JSONObject();
//            Iterator it = map.keySet().iterator();
//            while (it.hasNext()) {
//                String key = (String) it.next();
//                maps.put(key, map.get(key));
//            }
            JSONArray jsonArray =JSONArray.parseArray(maps.getString("array"));
            if (jsonArray.size()>0){
                jsonArray.sort(Comparator.comparing(obj->((JSONObject)obj).getDate("mergeTime"),Comparator.nullsFirst(Comparator.naturalOrder())));
                FusionData data = new FusionData();
                Integer num = 0;
                for (int i = 0; i < jsonArray.size(); i++) {
                    if (i<jsonArray.size()-1){
                        JSONObject job = jsonArray.getJSONObject(i);
                        JSONObject job2 = jsonArray.getJSONObject(i+1);
                        Date date = null;
                        Date date2 = null;
                        try {
                            date = DateUtils.parse(job.get("mergeTime").toString(),"yyyy-MM-dd HH:mm:ss.SSS");
                            date2 = DateUtils.parse(job2.get("mergeTime").toString(),"yyyy-MM-dd HH:mm:ss.SSS");
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                        Long interval = dateUtils(date2.getTime(),date.getTime());
                        if (interval>6500L){
                            num++;
                        }
                    }
                }
                if (num>0){
                    JSONObject sourceItem = jsonArray.getJSONObject(0);
                    JSONObject sourceItem2 = jsonArray.getJSONObject(jsonArray.size()-1);
                    data.setMergeTarget(sourceItem.getString("mergeTarget"));
                    data.setTrackCount((long) jsonArray.size());
                    Date date = null;
                    Date date2 = null;
                    try {
                        date = DateUtils.parse(sourceItem.getString("mergeTime"),"yyyy-MM-dd HH:mm:ss.SSS");
                        date2 = DateUtils.parse(sourceItem2.getString("mergeTime"),"yyyy-MM-dd HH:mm:ss.SSS");
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }
                    if (date.getTime()>date2.getTime()){
                        data.setStartTime(sourceItem2.getString("mergeTime"));
                        data.setEndTime(sourceItem.getString("mergeTime"));
                    }else {
                        data.setStartTime(sourceItem.getString("mergeTime"));
                        data.setEndTime(sourceItem2.getString("mergeTime"));
                    }
                    data.setFluctuateNumber(num);
                    list.add(data);
                }
            }
        });
//        Aggregations aggregations = search.getAggregations();
//        ParsedStringTerms aggregation = aggregations.get("ship_field_agg");
//        for (Terms.Bucket bucket : aggregation.getBuckets()) {
//            FusionData data = new FusionData();
//            ParsedTopHits topHits = bucket.getAggregations().get("ship_field_agg_top1");
//            SearchHits searchHits = topHits.getHits();
//            Long value = searchHits.getTotalHits().value;
//            SearchHit[] hits = searchHits.getHits();
//            Integer num = 0;
//            for (int i = 0; i < hits.length; i++) {
//                if (i != hits.length-1){
//                    Map<String, Object> map = hits[i].getSourceAsMap();
//                    Map<String, Object> map2 = hits[i+1].getSourceAsMap();
//                    Date date = DateUtils.parse(map.get("mergeTime").toString(),"yyyy-MM-dd");
//                    Date date2 = DateUtils.parse(map2.get("mergeTime").toString(),"yyyy-MM-dd");
//                    Long interval = date2.getTime()-date.getTime();
//                    if (interval>6500L){
//                        num++;
//                    }
//                }
//            }
//            if (num>0){
//                JSONObject sourceItem = JSONObject.parseObject(hits[0].getSourceAsString());
//                JSONObject sourceItem2 = JSONObject.parseObject(hits[value.intValue()-1].getSourceAsString());
//                data.setMergeTarget(sourceItem.getString("mergeTarget"));
//                data.setTrackCount(value);
//                data.setStartTime(sourceItem.getString("mergeTime"));
//                data.setEndTime(sourceItem2.getString("mergeTime"));
//                data.setFluctuateNumber(num);
//                list.add(data);
//            }
//        }
        try {
            System.out.println("进入导出识别融合数据率波动问题");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("识别融合数据率波动问题", "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), FusionData.class).sheet("识别融合数据率波动问题")
                    .doWrite(list);
        } catch (Exception e) {
            // 重置response
            System.out.println("导出识别融合数据率波动问题excel失败：{}" + e.getMessage() + e);
        }
    }

    @Override
    public void fusionAnalysisExcelTwo(FusionQuery query, HttpServletResponse response) throws IOException, ParseException {

        List<FusionData> list = new CopyOnWriteArrayList<>();
        Date startTime = DateUtils.parse(query.getStartTime(),"yyyy-MM-dd");
        String time = DateUtils.format(startTime,"yyyy-MM-dd");
        SearchRequest request = new SearchRequest(INDEX_SEAT_FK_REALTIMETRAJECTORY+time);
        // 构建查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("mergeTime").gte(query.getStartTime()).lte(query.getEndTime()));
        double[] doubles = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
        List<GeoPoint> points = new ArrayList<>();
        for(int i = 0 ;i < doubles.length;i=i+2) {
            points.add(new GeoPoint(doubles[i+1],doubles[i]));
        }
        boolQueryBuilder.filter(QueryBuilders.geoPolygonQuery("location", points));
        searchSourceBuilder.query(boolQueryBuilder);
//        TermsAggregationBuilder aggBuilder = AggregationBuilders.terms("ship_field_agg")
//                .field("mergeTarget")
//                .size(query.getPageSize())
//                .subAggregation(
//                        AggregationBuilders.terms("ship_field_agg_one")
//                                .field("targetCourse")
//                                .size(query.getPageSize())
//                                .subAggregation(
//                                        AggregationBuilders.terms("ship_field_agg_two")
//                                                .field("targetSpeed")
//                                                .size(query.getPageSize())
//                                                .subAggregation(AggregationBuilders.topHits("ship_field_agg_top1")
//                                                        .size(100)
//                                                        .sort(
//                                                                SortBuilders.fieldSort("mergeTime").order(SortOrder.ASC)
//                                                        ))
//                                )
//                );
        searchSourceBuilder
                .trackTotalHits(true)
//                .aggregation(aggBuilder)
                .size(query.getPageSize())
                .sort(SortBuilders.fieldSort("mergeTime").order(SortOrder.ASC))
                .timeout(TimeValue.timeValueHours(1L))
                .timeout(TimeValue.timeValueMinutes(30L))
                .timeout(TimeValue.timeValueSeconds(500L));
        request.source(searchSourceBuilder);
        SearchResponse search = client.search(request, COMMON_OPTIONS);
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        List<Map<String, Object>> inList = new CopyOnWriteArrayList<Map<String, Object>>();
        List<Map<String, Object>> outList = new CopyOnWriteArrayList<Map<String, Object>>();
        for (SearchHit hit : hits) {
            Map<String, Object> map = new ConcurrentHashMap();
            map = hit.getSourceAsMap();
            inList.add(map);
        }
        outList = change(inList, "mergeTarget", outList);
        outList.forEach(map->{
            JSONObject maps = JSONObject.parseObject(JSON.toJSONString(map));
//            JSONObject maps = new JSONObject();
//            Iterator it = map.keySet().iterator();
//            while (it.hasNext()) {
//                String key = (String) it.next();
//                maps.put(key, map.get(key));
//            }
            FusionData data = new FusionData();
            JSONArray jsonArray = JSONArray.parseArray(maps.getString("array"));
            Integer num = 0;
            if (jsonArray.size() > 0) {
                for (int i = 0; i < jsonArray.size(); i++) {
                    JSONObject job = jsonArray.getJSONObject(i);
                    for (int j = 0; j+i < jsonArray.size(); j++){
                        JSONObject job2 = jsonArray.getJSONObject(j+i);
                        if (job.getString("targetCourse").equals(job2.getString("targetCourse"))){
                            if (job.getString("targetSpeed").equals(job2.getString("targetSpeed"))){
                                if (num==3){
                                    if (data.getFluctuateNumber()!=null){
                                        data.setFluctuateNumber(data.getFluctuateNumber()+1);
                                    }else {
                                        data.setFluctuateNumber(1);
                                    }
                                    data.setMergeTarget(maps.getString("text"));
                                    data.setTrackCount((long) jsonArray.size());
                                    String start=jsonArray.getJSONObject(0).getString("mergeTime");
                                    String end=jsonArray.getJSONObject(jsonArray.size()-1).getString("mergeTime");
                                    Date date = null;
                                    Date date2 = null;
                                    try {
                                        date = DateUtils.parse(start,"yyyy-MM-dd HH:mm:ss.SSS");
                                        date2 = DateUtils.parse(end,"yyyy-MM-dd HH:mm:ss.SSS");
                                    } catch (ParseException e) {
                                        e.printStackTrace();
                                    }
                                    if (date.getTime()>date2.getTime()){
                                        data.setStartTime(end);
                                        data.setEndTime(start);
                                    }else {
                                        data.setStartTime(start);
                                        data.setEndTime(end);
                                    }
                                    list.add(data);
                                    num =0;
                                }
                                Date date = null;
                                Date date2 = null;
                                try {
                                    date = DateUtils.parse(job.getString("mergeTime"),"yyyy-MM-dd HH:mm:ss.SSS");
                                    date2 = DateUtils.parse(job2.getString("mergeTime"),"yyyy-MM-dd HH:mm:ss.SSS");
                                } catch (ParseException e) {
                                    e.printStackTrace();
                                }
                                Long interval = dateUtils(date.getTime(),date2.getTime());
                                if (interval<6500L){
                                    num++;
                                }
                            }
                        }
                    }
                }
            }
        });
//        Aggregations aggregations = search.getAggregations();
//        ParsedStringTerms aggregation = aggregations.get("ship_field_agg");
//        for (Terms.Bucket bucket : aggregation.getBuckets()) {
//            FusionData data = new FusionData();
//            ParsedStringTerms terms = bucket.getAggregations().get("ship_field_agg_one");
//            Integer num = 0;
//            for (Terms.Bucket buc : terms.getBuckets()){
//                ParsedDoubleTerms termsTwo = buc.getAggregations().get("ship_field_agg_two");
//                for (Terms.Bucket ket : termsTwo.getBuckets()){
//                    ParsedTopHits topHit = ket.getAggregations().get("ship_field_agg_top1");
//                    SearchHits searchHit = topHit.getHits();
//                    SearchHit[] hit = searchHit.getHits();
//                    for (int i = 0; i < hit.length; i++) {
//                        if (num>2){
//                            data.setFluctuateNumber(data.getFluctuateNumber()+1);
//                            data.setMergeTarget(bucket.getKeyAsString());
//                            data.setTrackCount(bucket.getDocCount());
//                            data.setStartTime(query.getStartTime());
//                            data.setEndTime(query.getEndTime());
//                            list.add(data);
//                            num =0;
//                        }
//                        if (i != hit.length-1){
//                            Map<String, Object> map = hit[i].getSourceAsMap();
//                            Map<String, Object> map2 = hit[i+1].getSourceAsMap();
//                            Date date = DateUtils.parse(map.get("mergeTime").toString(),"yyyy-MM-dd");
//                            Date date2 = DateUtils.parse(map2.get("mergeTime").toString(),"yyyy-MM-dd");
//                            Long interval = date2.getTime()-date.getTime();
//                            if (interval<6500L){
//                                num++;
//                            }
//                        }
//                    }
//                }
//            }
//        }
        try {
            System.out.println("进入导出虚假推送问题");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("虚假推送问题", "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), FusionData.class).sheet("虚假推送问题")
                    .doWrite(list);
        } catch (Exception e) {
            // 重置response
            System.out.println("导出虚假推送问题excel失败：{}" + e.getMessage() + e);
        }
    }

    @Override
    public void tianAoLeaveOutExcel(FusionQuery query, HttpServletResponse response) throws IOException, ParseException {
        List<LeaveOutData> list = new CopyOnWriteArrayList<>();
        List<LeaceOutDto> dtos = new CopyOnWriteArrayList<>();
        Date startTime = DateUtils.parse(query.getStartTime(),"yyyy-MM-dd");
        String time = DateUtils.format(startTime,"yyyy-MM-dd");
        SearchRequest request = new SearchRequest(INDEX_SEAT_TRACK_RADAR+time);
        // 构建查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("receiveTime").gte(query.getStartTime()).lte(query.getEndTime()));
        double[] doubles = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
        List<GeoPoint> points = new ArrayList<>();
        for(int i = 0 ;i < doubles.length;i=i+2) {
            points.add(new GeoPoint(doubles[i+1],doubles[i]));
        }
        boolQueryBuilder.filter(QueryBuilders.geoPolygonQuery("location", points));
        searchSourceBuilder.query(boolQueryBuilder);
//        TermsAggregationBuilder aggBuilder = AggregationBuilders.terms("ship_field_agg")
//                .field("fusionBatchNum")
//                .size(query.getPageSize());
        searchSourceBuilder
                .trackTotalHits(true)
//                .aggregation(aggBuilder)
                .size(query.getPageSize())
                .sort(SortBuilders.fieldSort("receiveTime").order(SortOrder.ASC))
                .timeout(TimeValue.timeValueHours(1L))
                .timeout(TimeValue.timeValueMinutes(30L))
                .timeout(TimeValue.timeValueSeconds(500L));
        request.source(searchSourceBuilder);
        SearchResponse search = client.search(request, COMMON_OPTIONS);
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        List<Map<String, Object>> inList = new CopyOnWriteArrayList<Map<String, Object>>();
        List<Map<String, Object>> outList = new CopyOnWriteArrayList<Map<String, Object>>();
        for (SearchHit hit : hits) {
            Map<String, Object> map = new ConcurrentHashMap();
            map = hit.getSourceAsMap();
            inList.add(map);
        }
        outList = change(inList, "fusionBatchNum", outList);
        outList.forEach(map-> {
            JSONObject maps = JSONObject.parseObject(JSON.toJSONString(map));
//            JSONObject maps = new JSONObject();
//            Iterator it = map.keySet().iterator();
//            while (it.hasNext()) {
//                String key = (String) it.next();
//                maps.put(key, map.get(key));
//            }
            JSONArray jsonArray = JSONArray.parseArray(maps.getString("array"));
            if (jsonArray.size() > 0) {
                JSONObject job = jsonArray.getJSONObject(0);
                LeaceOutDto dto = new LeaceOutDto();
                dto.setBatchNumber(job.getString("fusionBatchNum"));
                dto.setCount((long) jsonArray.size());
                dtos.add(dto);
            }
        });
//        Aggregations aggregations = search.getAggregations();
//        ParsedStringTerms aggregation = aggregations.get("ship_field_agg");
//        for (Terms.Bucket bucket : aggregation.getBuckets()) {
//            LeaceOutDto dto = new LeaceOutDto();
//            dto.setBatchNumber(bucket.getKeyAsString());
//            dto.setCount(bucket.getDocCount());
//            dtos.add(dto);
//        }
        dtos.forEach(item->{
            LeaveOutData data = new LeaveOutData();
            SearchRequest request2 = new SearchRequest(INDEX_SEAT_FK_REALTIMETRAJECTORY+time);
            // 构建查询条件
            BoolQueryBuilder boolQueryBuilder2 = QueryBuilders.boolQuery();
            SearchSourceBuilder searchSourceBuilder2 = new SearchSourceBuilder();
            boolQueryBuilder2.filter(QueryBuilders.wildcardQuery("trackDeviceNo", "*,"+ item.getBatchNumber() +",*"));
            boolQueryBuilder2.filter(QueryBuilders.rangeQuery("mergeTime").gte(query.getStartTime()).lte(query.getEndTime()));
            boolQueryBuilder2.filter(QueryBuilders.termsQuery("mergeType","TA_RADAR"));
            double[] doubles2 = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
            List<GeoPoint> points2 = new ArrayList<>();
            for(int i = 0 ;i < doubles2.length;i=i+2) {
                points2.add(new GeoPoint(doubles2[i+1],doubles2[i]));
            }
            boolQueryBuilder2.filter(QueryBuilders.geoPolygonQuery("location", points2));
            searchSourceBuilder2.query(boolQueryBuilder2)
                    .trackTotalHits(true)
                    .size(query.getPageSize())
                    .timeout(TimeValue.timeValueHours(1L))
                    .timeout(TimeValue.timeValueMinutes(30L))
                    .timeout(TimeValue.timeValueSeconds(500L));
            request2.source(searchSourceBuilder2);
            try {
                SearchResponse search2 = client.search(request2, COMMON_OPTIONS);
                SearchHits searchHits2 = search2.getHits();
                Long totalHits = searchHits2.getTotalHits().value;
                if (totalHits<5) {
                    SearchHit[] hits2 = searchHits2.getHits();
                    if (hits2.length > 0) {
                        Map<String, Object> map = hits2[0].getSourceAsMap();
                        Map<String, Object> map2 = hits2[hits2.length-1].getSourceAsMap();
                        data.setMergeTarget(map.get("mergeTarget").toString());
                        Date date = DateUtils.parse(map.get("mergeTime").toString(),"yyyy-MM-dd HH:mm:ss.SSS");
                        Date date2 = DateUtils.parse(map2.get("mergeTime").toString(),"yyyy-MM-dd HH:mm:ss.SSS");
                        if (date.getTime()>date2.getTime()){
                            data.setStartTime(map2.get("mergeTime").toString());
                            data.setEndTime(map.get("mergeTime").toString());
                        }else {
                            data.setStartTime(map.get("mergeTime").toString());
                            data.setEndTime(map2.get("mergeTime").toString());
                        }
                        data.setBatchNumber(item.getBatchNumber());
                        data.setOriginalCount(item.getCount());
                        data.setTrackCount(totalHits);
                        list.add(data);
                    }
                }
            } catch (IOException | ParseException e) {
                e.printStackTrace();
            }
        });
        try {
            System.out.println("进入导出天奥轨迹数据遗漏");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("天奥轨迹数据遗漏", "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), LeaveOutData.class).sheet("天奥轨迹数据遗漏")
                    .doWrite(list);
        } catch (Exception e) {
            // 重置response
            System.out.println("导出天奥轨迹数据遗漏excel失败：{}" + e.getMessage() + e);
        }
    }

    @Override
    public void beiDouLeaveOutExcel(FusionQuery query, HttpServletResponse response) throws IOException, ParseException {
        List<LeaveOutData> list = new CopyOnWriteArrayList<>();
        List<LeaceOutDto> dtos = new CopyOnWriteArrayList<>();
        Date startTime = DateUtils.parse(query.getStartTime(),"yyyy-MM-dd");
        String time = DateUtils.format(startTime,"yyyy-MM-dd");
        SearchRequest request = new SearchRequest(INDEX_SEAT_TRACK_BEIDOU+time);
        // 构建查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("locationTime").gte(query.getStartTime()).lte(query.getEndTime()));
        double[] doubles = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
        List<GeoPoint> points = new ArrayList<>();
        for(int i = 0 ;i < doubles.length;i=i+2) {
            points.add(new GeoPoint(doubles[i+1],doubles[i]));
        }
        boolQueryBuilder.filter(QueryBuilders.geoPolygonQuery("location", points));
        searchSourceBuilder.query(boolQueryBuilder);
//        TermsAggregationBuilder aggBuilder = AggregationBuilders.terms("ship_field_agg")
//                .field("fusionBatchNum")
//                .size(query.getPageSize());
        searchSourceBuilder
                .trackTotalHits(true)
//                .aggregation(aggBuilder)
                .size(query.getPageSize())
                .sort(SortBuilders.fieldSort("locationTime").order(SortOrder.ASC))
                .timeout(TimeValue.timeValueHours(1L))
                .timeout(TimeValue.timeValueMinutes(30L))
                .timeout(TimeValue.timeValueSeconds(500L));
        request.source(searchSourceBuilder);
        SearchResponse search = client.search(request, COMMON_OPTIONS);
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        List<Map<String, Object>> inList = new CopyOnWriteArrayList<Map<String, Object>>();
        List<Map<String, Object>> outList = new CopyOnWriteArrayList<Map<String, Object>>();
        for (SearchHit hit : hits) {
            Map<String, Object> map = new ConcurrentHashMap();
            map = hit.getSourceAsMap();
            inList.add(map);
        }
        outList = change(inList, "devideNo", outList);
        outList.forEach(map-> {
            JSONObject maps = JSONObject.parseObject(JSON.toJSONString(map));
//            JSONObject maps = new JSONObject();
//            Iterator it = map.keySet().iterator();
//            while (it.hasNext()) {
//                String key = (String) it.next();
//                maps.put(key, map.get(key));
//            }
            JSONArray jsonArray = JSONArray.parseArray(maps.getString("array"));
            if (jsonArray.size() > 0) {
                JSONObject job = jsonArray.getJSONObject(0);
                LeaceOutDto dto = new LeaceOutDto();
                dto.setBatchNumber(job.getString("devideNo"));
                dto.setCount((long) jsonArray.size());
                dtos.add(dto);
            }
        });
//        Aggregations aggregations = search.getAggregations();
//        ParsedStringTerms aggregation = aggregations.get("ship_field_agg");
//        for (Terms.Bucket bucket : aggregation.getBuckets()) {
//            LeaceOutDto dto = new LeaceOutDto();
//            dto.setBatchNumber(bucket.getKeyAsString());
//            dto.setCount(bucket.getDocCount());
//            dtos.add(dto);
//        }
        dtos.forEach(item->{
            LeaveOutData data = new LeaveOutData();
            SearchRequest request2 = new SearchRequest(INDEX_SEAT_FK_REALTIMETRAJECTORY+time);
            // 构建查询条件
            BoolQueryBuilder boolQueryBuilder2 = QueryBuilders.boolQuery();
            SearchSourceBuilder searchSourceBuilder2 = new SearchSourceBuilder();
            boolQueryBuilder2.filter(QueryBuilders.wildcardQuery("trackDeviceNo", "*,"+ item.getBatchNumber() +",*"));
            boolQueryBuilder2.filter(QueryBuilders.rangeQuery("mergeTime").gte(query.getStartTime()).lte(query.getEndTime()));
            boolQueryBuilder2.filter(QueryBuilders.termsQuery("mergeType","BEIDOU","AIS_BEIDOU","BEIDOU_RADAR","AIS_BEIDOU_RADAR"));
            double[] doubles2 = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
            List<GeoPoint> points2 = new ArrayList<>();
            for(int i = 0 ;i < doubles2.length;i=i+2) {
                points2.add(new GeoPoint(doubles2[i+1],doubles2[i]));
            }
            boolQueryBuilder2.filter(QueryBuilders.geoPolygonQuery("location", points2));
            searchSourceBuilder2.query(boolQueryBuilder2)
                    .trackTotalHits(true)
                    .size(query.getPageSize())
                    .timeout(TimeValue.timeValueHours(1L))
                    .timeout(TimeValue.timeValueMinutes(30L))
                    .timeout(TimeValue.timeValueSeconds(500L));
            request2.source(searchSourceBuilder2);
            try {
                SearchResponse search2 = client.search(request2, COMMON_OPTIONS);
                SearchHits searchHits2 = search2.getHits();
                Long totalHits = searchHits2.getTotalHits().value;
                if (totalHits<5) {
                    SearchHit[] hits2 = searchHits2.getHits();
                    if (hits2.length > 0) {
                        Map<String, Object> map = hits2[0].getSourceAsMap();
                        Map<String, Object> map2 = hits2[hits2.length-1].getSourceAsMap();
                        data.setMergeTarget(map.get("mergeTarget").toString());
                        Date date = DateUtils.parse(map.get("mergeTime").toString(),"yyyy-MM-dd HH:mm:ss.SSS");
                        Date date2 = DateUtils.parse(map2.get("mergeTime").toString(),"yyyy-MM-dd HH:mm:ss.SSS");
                        if (date.getTime()>date2.getTime()){
                            data.setStartTime(map2.get("mergeTime").toString());
                            data.setEndTime(map.get("mergeTime").toString());
                        }else {
                            data.setStartTime(map.get("mergeTime").toString());
                            data.setEndTime(map2.get("mergeTime").toString());
                        }
                        data.setBatchNumber(item.getBatchNumber());
                        data.setOriginalCount(item.getCount());
                        data.setTrackCount(totalHits);
                        list.add(data);
                    }
                }
            } catch (IOException | ParseException e) {
                e.printStackTrace();
            }
        });
        try {
            System.out.println("进入导出北斗轨迹数据遗漏");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("北斗轨迹数据遗漏", "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), LeaveOutData.class).sheet("北斗轨迹数据遗漏")
                    .doWrite(list);
        } catch (Exception e) {
            // 重置response
            System.out.println("导出北斗轨迹数据遗漏excel失败：{}" + e.getMessage() + e);
        }
    }

    @Override
    public void hlxyjrhLeaveOutExcel(FusionQuery query, HttpServletResponse response) throws IOException, ParseException {
        List<LeaveOutData> list = new CopyOnWriteArrayList<>();
        List<LeaceOutDto> dtos = new CopyOnWriteArrayList<>();
        Date startTime = DateUtils.parse(query.getStartTime(),"yyyy-MM-dd");
        String time = DateUtils.format(startTime,"yyyy-MM-dd");
        SearchRequest request = new SearchRequest(INDEX_SEAT_HLX_FUSION_SHIP+time);
        // 构建查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("time").gte(query.getStartTime()).lte(query.getEndTime()));
        double[] doubles = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
        List<GeoPoint> points = new ArrayList<>();
        for(int i = 0 ;i < doubles.length;i=i+2) {
            points.add(new GeoPoint(doubles[i+1],doubles[i]));
        }
        boolQueryBuilder.filter(QueryBuilders.geoPolygonQuery("location", points));
        searchSourceBuilder.query(boolQueryBuilder);
//        TermsAggregationBuilder aggBuilder = AggregationBuilders.terms("ship_field_agg")
//                .field("fusionBatchNum")
//                .size(query.getPageSize());
        searchSourceBuilder
                .trackTotalHits(true)
//                .aggregation(aggBuilder)
                .size(query.getPageSize())
                .sort(SortBuilders.fieldSort("time").order(SortOrder.ASC))
                .timeout(TimeValue.timeValueHours(1L))
                .timeout(TimeValue.timeValueMinutes(30L))
                .timeout(TimeValue.timeValueSeconds(500L));
        request.source(searchSourceBuilder);
        SearchResponse search = client.search(request, COMMON_OPTIONS);
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        List<Map<String, Object>> inList = new CopyOnWriteArrayList<Map<String, Object>>();
        List<Map<String, Object>> outList = new CopyOnWriteArrayList<Map<String, Object>>();
        for (SearchHit hit : hits) {
            Map<String, Object> map = new ConcurrentHashMap();
            map = hit.getSourceAsMap();
            inList.add(map);
        }
        outList = change(inList, "targetID", outList);
        outList.forEach(map-> {
            JSONObject maps = JSONObject.parseObject(JSON.toJSONString(map));
//            JSONObject maps = new JSONObject();
//            Iterator it = map.keySet().iterator();
//            while (it.hasNext()) {
//                String key = (String) it.next();
//                maps.put(key, map.get(key));
//            }
            JSONArray jsonArray = JSONArray.parseArray(maps.getString("array"));
            if (jsonArray.size() > 0) {
                JSONObject job = jsonArray.getJSONObject(0);
                LeaceOutDto dto = new LeaceOutDto();
                dto.setBatchNumber(job.getString("targetID"));
                dto.setCount((long) jsonArray.size());
                dtos.add(dto);
            }
        });
//        Aggregations aggregations = search.getAggregations();
//        ParsedStringTerms aggregation = aggregations.get("ship_field_agg");
//        for (Terms.Bucket bucket : aggregation.getBuckets()) {
//            LeaceOutDto dto = new LeaceOutDto();
//            dto.setBatchNumber(bucket.getKeyAsString());
//            dto.setCount(bucket.getDocCount());
//            dtos.add(dto);
//        }
        dtos.forEach(item->{
            LeaveOutData data = new LeaveOutData();
            SearchRequest request2 = new SearchRequest(INDEX_SEAT_FK_REALTIMETRAJECTORY+time);
            // 构建查询条件
            BoolQueryBuilder boolQueryBuilder2 = QueryBuilders.boolQuery();
            SearchSourceBuilder searchSourceBuilder2 = new SearchSourceBuilder();
            boolQueryBuilder2.filter(QueryBuilders.wildcardQuery("trackDeviceNo", "*,"+ item.getBatchNumber() +",*"));
            boolQueryBuilder2.filter(QueryBuilders.rangeQuery("mergeTime").gte(query.getStartTime()).lte(query.getEndTime()));
            boolQueryBuilder2.filter(QueryBuilders.termsQuery("mergeType","HLX_AIS_RADAR","HLX_AIS","HLX_RADAR","HLX_ZYD_RADAR"));
            double[] doubles2 = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
            List<GeoPoint> points2 = new ArrayList<>();
            for(int i = 0 ;i < doubles2.length;i=i+2) {
                points2.add(new GeoPoint(doubles2[i+1],doubles2[i]));
            }
            boolQueryBuilder2.filter(QueryBuilders.geoPolygonQuery("location", points2));
            searchSourceBuilder2.query(boolQueryBuilder2)
                    .trackTotalHits(true)
                    .size(query.getPageSize())
                    .timeout(TimeValue.timeValueHours(1L))
                    .timeout(TimeValue.timeValueMinutes(30L))
                    .timeout(TimeValue.timeValueSeconds(500L));
            request2.source(searchSourceBuilder2);
            try {
                SearchResponse search2 = client.search(request2, COMMON_OPTIONS);
                SearchHits searchHits2 = search2.getHits();
                Long totalHits = searchHits2.getTotalHits().value;
                if (totalHits<5) {
                    SearchHit[] hits2 = searchHits2.getHits();
                    if (hits2.length > 0) {
                        Map<String, Object> map = hits2[0].getSourceAsMap();
                        Map<String, Object> map2 = hits2[hits2.length-1].getSourceAsMap();
                        data.setMergeTarget(map.get("mergeTarget").toString());
                        Date date = DateUtils.parse(map.get("mergeTime").toString(),"yyyy-MM-dd HH:mm:ss.SSS");
                        Date date2 = DateUtils.parse(map2.get("mergeTime").toString(),"yyyy-MM-dd HH:mm:ss.SSS");
                        if (date.getTime()>date2.getTime()){
                            data.setStartTime(map2.get("mergeTime").toString());
                            data.setEndTime(map.get("mergeTime").toString());
                        }else {
                            data.setStartTime(map.get("mergeTime").toString());
                            data.setEndTime(map2.get("mergeTime").toString());
                        }
                        data.setBatchNumber(item.getBatchNumber());
                        data.setOriginalCount(item.getCount());
                        data.setTrackCount(totalHits);
                        list.add(data);
                    }
                }
            } catch (IOException | ParseException e) {
                e.printStackTrace();
            }
        });
        try {
            System.out.println("进入导出海兰信1级融合轨迹数据遗漏");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("海兰信1级融合轨迹数据遗漏", "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), LeaveOutData.class).sheet("海兰信1级融合轨迹数据遗漏")
                    .doWrite(list);
        } catch (Exception e) {
            // 重置response
            System.out.println("导出海兰信1级融合轨迹数据遗漏excel失败：{}" + e.getMessage() + e);
        }
    }

    @Override
    public void tianaoStaticOrDynamicLeaveOutExcel(FusionQuery query, HttpServletResponse response) throws IOException, ParseException {
        List<StaticOrDynamicData> list = new CopyOnWriteArrayList<>();
        List<LeaceOutDto> dtos = new CopyOnWriteArrayList<>();
        Date startTime = DateUtils.parse(query.getStartTime(),"yyyy-MM-dd");
        String time = DateUtils.format(startTime,"yyyy-MM-dd");
        SearchRequest request = new SearchRequest(INDEX_SEAT_TRACK_RADAR+time);
        // 构建查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("receiveTime").gte(query.getStartTime()).lte(query.getEndTime()));
        boolQueryBuilder.mustNot(QueryBuilders.termQuery("targetProper","0"));
        double[] doubles = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
        List<GeoPoint> points = new ArrayList<>();
        for(int i = 0 ;i < doubles.length;i=i+2) {
            points.add(new GeoPoint(doubles[i+1],doubles[i]));
        }
        boolQueryBuilder.filter(QueryBuilders.geoPolygonQuery("location", points));
        searchSourceBuilder.query(boolQueryBuilder);
//        TermsAggregationBuilder aggBuilder = AggregationBuilders.terms("ship_field_agg")
//                .field("fusionBatchNum")
//                .size(query.getPageSize());
        searchSourceBuilder
                .trackTotalHits(true)
//                .aggregation(aggBuilder)
                .size(query.getPageSize())
                .timeout(TimeValue.timeValueHours(1L))
                .timeout(TimeValue.timeValueMinutes(30L))
                .timeout(TimeValue.timeValueSeconds(500L));
        request.source(searchSourceBuilder);
        SearchResponse search = client.search(request, COMMON_OPTIONS);
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        List<Map<String, Object>> inList = new CopyOnWriteArrayList<Map<String, Object>>();
        List<Map<String, Object>> outList = new CopyOnWriteArrayList<Map<String, Object>>();
        for (SearchHit hit : hits) {
            Map<String, Object> map = new ConcurrentHashMap();
            map = hit.getSourceAsMap();
            inList.add(map);
        }
        outList = change(inList, "fusionBatchNum", outList);
        outList.forEach(map-> {
            JSONObject maps = JSONObject.parseObject(JSON.toJSONString(map));
//            JSONObject maps = new JSONObject();
//            Iterator it = map.keySet().iterator();
//            while (it.hasNext()) {
//                String key = (String) it.next();
//                maps.put(key, map.get(key));
//            }
            JSONArray jsonArray = JSONArray.parseArray(maps.getString("array"));
            if (jsonArray.size() > 0) {
                JSONObject job = jsonArray.getJSONObject(0);
                LeaceOutDto dto = new LeaceOutDto();
                dto.setBatchNumber(job.getString("fusionBatchNum"));
                dto.setCount((long) jsonArray.size());
                dtos.add(dto);
            }
        });
        dtos.forEach(item->{
            StaticOrDynamicData data = new StaticOrDynamicData();
            SearchRequest request2 = new SearchRequest(INDEX_SEAT_FK_REALTIMETRAJECTORY+time);
            // 构建查询条件
            BoolQueryBuilder boolQueryBuilder2 = QueryBuilders.boolQuery();
            SearchSourceBuilder searchSourceBuilder2 = new SearchSourceBuilder();
            boolQueryBuilder2.filter(QueryBuilders.wildcardQuery("trackDeviceNo", "*,"+ item.getBatchNumber() +",*"));
            boolQueryBuilder2.filter(QueryBuilders.termQuery("targetType","0"));
            boolQueryBuilder2.filter(QueryBuilders.rangeQuery("mergeTime").gte(query.getStartTime()).lte(query.getEndTime()));
            boolQueryBuilder2.filter(QueryBuilders.termsQuery("mergeType","RADAR","AIS_RADAR","BEIDOU_RADAR","AIS_BEIDOU_RADAR"));
            double[] doubles2 = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
            List<GeoPoint> points2 = new ArrayList<>();
            for(int i = 0 ;i < doubles2.length;i=i+2) {
                points2.add(new GeoPoint(doubles2[i+1],doubles2[i]));
            }
            boolQueryBuilder2.filter(QueryBuilders.geoPolygonQuery("location", points2));
            searchSourceBuilder2.query(boolQueryBuilder2)
                    .trackTotalHits(true)
                    .size(query.getPageSize())
                    .timeout(TimeValue.timeValueHours(1L))
                    .timeout(TimeValue.timeValueMinutes(30L))
                    .timeout(TimeValue.timeValueSeconds(500L));
            request2.source(searchSourceBuilder2);
            try {
                SearchResponse search2 = client.search(request2, COMMON_OPTIONS);
                SearchHits searchHits2 = search2.getHits();
                SearchHit[] hits2 = searchHits2.getHits();
                if (hits2.length > 0) {
                    JSONObject sourceItem = JSONObject.parseObject(hits2[0].getSourceAsString());
                    data.setMergeTarget(sourceItem.getString("mergeTarget"));
                    data.setBatchNumber(item.getBatchNumber());
                    list.add(data);
                }
//                else {
//                    data.setMergeTarget("未查出匹配融合数据");
//                    data.setBatchNumber(item.getBatchNumber());
//                    list.add(data);
//                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        });
        try {
            System.out.println("进入导出天奥静态数据与动态轨迹融合");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("天奥静态数据与动态轨迹融合", "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), StaticOrDynamicData.class).sheet("天奥静态数据与动态轨迹融合")
                    .doWrite(list);
        } catch (Exception e) {
            // 重置response
            System.out.println("导出天奥静态数据与动态轨迹融合excel失败：{}" + e.getMessage() + e);
        }
    }

    @Override
    public void fusionAnalysisExcelThree(FusionQuery query, HttpServletResponse response) throws IOException, ParseException {
        List<FusionData> list = new CopyOnWriteArrayList<>();
        Date startTime = DateUtils.parse(query.getStartTime(),"yyyy-MM-dd");
        String time = DateUtils.format(startTime,"yyyy-MM-dd");
        SearchRequest request = new SearchRequest(INDEX_SEAT_FK_REALTIMETRAJECTORY+time);
        // 构建查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("mergeTime").gte(query.getStartTime()).lte(query.getEndTime()));
        double[] doubles = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
        List<GeoPoint> points = new ArrayList<>();
        for(int i = 0 ;i < doubles.length;i=i+2) {
            points.add(new GeoPoint(doubles[i+1],doubles[i]));
        }
        boolQueryBuilder.filter(QueryBuilders.geoPolygonQuery("location", points));
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder
                .trackTotalHits(true)
                .sort(SortBuilders.fieldSort("mergeTime").order(SortOrder.ASC))
                .size(query.getPageSize())
                .timeout(TimeValue.timeValueHours(1L))
                .timeout(TimeValue.timeValueMinutes(30L))
                .timeout(TimeValue.timeValueSeconds(500L));
        request.source(searchSourceBuilder);
        SearchResponse search = client.search(request, COMMON_OPTIONS);
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        List<Map<String, Object>> inList = new CopyOnWriteArrayList<Map<String, Object>>();
        List<Map<String, Object>> outList = new CopyOnWriteArrayList<Map<String, Object>>();
        for (SearchHit hit : hits) {
            Map<String, Object> map = new ConcurrentHashMap();
            map = hit.getSourceAsMap();
            inList.add(map);
        }
        outList = change(inList, "mergeTarget", outList);
        outList.forEach(map->{
            JSONObject maps = JSONObject.parseObject(JSON.toJSONString(map));
            JSONArray jsonArray =JSONArray.parseArray(maps.getString("array"));
            if (jsonArray.size()>0){
                jsonArray.sort(Comparator.comparing(obj->((JSONObject)obj).getDate("mergeTime"),Comparator.nullsFirst(Comparator.naturalOrder())));
                FusionData data = new FusionData();
                Integer num = 0;
                for (int i = 0; i < jsonArray.size(); i++) {
                    if (i<jsonArray.size()-1){
                        JSONObject job = jsonArray.getJSONObject(i);
                        JSONObject job2 = jsonArray.getJSONObject(i+1);
                        String targetSpeed = job.get("targetSpeed").toString();
                        String targetSpeed2 = job2.get("targetSpeed").toString();
                        Double d= Double.parseDouble(targetSpeed);
                        Double d2= Double.parseDouble(targetSpeed2);
                        Double aDouble = speedUtils(d, d2);
                        if (aDouble>3.0D){
                            num++;
                        }
                    }
                }
                if (num>0){
                    JSONObject sourceItem = jsonArray.getJSONObject(0);
                    JSONObject sourceItem2 = jsonArray.getJSONObject(jsonArray.size()-1);
                    data.setMergeTarget(sourceItem.getString("mergeTarget"));
                    data.setTrackCount((long) jsonArray.size());
                    Date date = null;
                    Date date2 = null;
                    try {
                        date = DateUtils.parse(sourceItem.getString("mergeTime"),"yyyy-MM-dd HH:mm:ss.SSS");
                        date2 = DateUtils.parse(sourceItem2.getString("mergeTime"),"yyyy-MM-dd HH:mm:ss.SSS");
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }
                    if (date.getTime()>date2.getTime()){
                        data.setStartTime(sourceItem2.getString("mergeTime"));
                        data.setEndTime(sourceItem.getString("mergeTime"));
                    }else {
                        data.setStartTime(sourceItem.getString("mergeTime"));
                        data.setEndTime(sourceItem2.getString("mergeTime"));
                    }
                    data.setFluctuateNumber(num);
                    list.add(data);
                }
            }
        });
        try {
            System.out.println("进入导出识别融合速度异常波动问题");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("识别融合速度异常波动问题", "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), FusionData.class).sheet("识别融合速度异常波动问题")
                    .doWrite(list);
        } catch (Exception e) {
            // 重置response
            System.out.println("导出识别融合速度异常波动问题excel失败：{}" + e.getMessage() + e);
        }
    }

    @Override
    public void fusionAnalysisExcelFour(FusionQuery query, HttpServletResponse response) throws IOException, ParseException {
        List<FusionData> list = new CopyOnWriteArrayList<>();
        Date startTime = DateUtils.parse(query.getStartTime(),"yyyy-MM-dd");
        String time = DateUtils.format(startTime,"yyyy-MM-dd");
        SearchRequest request = new SearchRequest(INDEX_SEAT_FK_REALTIMETRAJECTORY+time);
        // 构建查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("mergeTime").gte(query.getStartTime()).lte(query.getEndTime()));
        double[] doubles = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
        List<GeoPoint> points = new ArrayList<>();
        for(int i = 0 ;i < doubles.length;i=i+2) {
            points.add(new GeoPoint(doubles[i+1],doubles[i]));
        }
        boolQueryBuilder.filter(QueryBuilders.geoPolygonQuery("location", points));
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder
                .trackTotalHits(true)
                .sort(SortBuilders.fieldSort("mergeTime").order(SortOrder.ASC))
                .size(query.getPageSize())
                .timeout(TimeValue.timeValueHours(1L))
                .timeout(TimeValue.timeValueMinutes(30L))
                .timeout(TimeValue.timeValueSeconds(500L));
        request.source(searchSourceBuilder);
        SearchResponse search = client.search(request, COMMON_OPTIONS);
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        List<Map<String, Object>> inList = new CopyOnWriteArrayList<Map<String, Object>>();
        List<Map<String, Object>> outList = new CopyOnWriteArrayList<Map<String, Object>>();
        for (SearchHit hit : hits) {
            Map<String, Object> map = new ConcurrentHashMap();
            map = hit.getSourceAsMap();
            inList.add(map);
        }
        outList = change(inList, "mergeTarget", outList);
        outList.forEach(map->{
            JSONObject maps = JSONObject.parseObject(JSON.toJSONString(map));
            JSONArray jsonArray =JSONArray.parseArray(maps.getString("array"));
            if (jsonArray.size()>0){
                jsonArray.sort(Comparator.comparing(obj->((JSONObject)obj).getDate("mergeTime"),Comparator.nullsFirst(Comparator.naturalOrder())));
                FusionData data = new FusionData();
                Integer num = 0;
                for (int i = 0; i < jsonArray.size(); i++) {
                    if (i<jsonArray.size()-1){
                        JSONObject job = jsonArray.getJSONObject(i);
                        JSONObject job2 = jsonArray.getJSONObject(i+1);
                        String targetCourse = job.get("targetCourse").toString();
                        String targetCourse2 = job2.get("targetCourse").toString();
                        Double d= Double.parseDouble(targetCourse);
                        Double d2= Double.parseDouble(targetCourse2);
                        Double aDouble = speedUtils(d, d2);
                        if (aDouble>60.0D){
                            num++;
                        }
                    }
                }
                if (num>0){
                    JSONObject sourceItem = jsonArray.getJSONObject(0);
                    JSONObject sourceItem2 = jsonArray.getJSONObject(jsonArray.size()-1);
                    data.setMergeTarget(sourceItem.getString("mergeTarget"));
                    data.setTrackCount((long) jsonArray.size());
                    Date date = null;
                    Date date2 = null;
                    try {
                        date = DateUtils.parse(sourceItem.getString("mergeTime"),"yyyy-MM-dd HH:mm:ss.SSS");
                        date2 = DateUtils.parse(sourceItem2.getString("mergeTime"),"yyyy-MM-dd HH:mm:ss.SSS");
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }
                    if (date.getTime()>date2.getTime()){
                        data.setStartTime(sourceItem2.getString("mergeTime"));
                        data.setEndTime(sourceItem.getString("mergeTime"));
                    }else {
                        data.setStartTime(sourceItem.getString("mergeTime"));
                        data.setEndTime(sourceItem2.getString("mergeTime"));
                    }
                    data.setFluctuateNumber(num);
                    list.add(data);
                }
            }
        });
        try {
            System.out.println("进入导出识别融合航向异常波动问题");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("识别融合数据率波动问题", "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), FusionData.class).sheet("识别融合航向异常波动问题")
                    .doWrite(list);
        } catch (Exception e) {
            // 重置response
            System.out.println("导出识别融合航向异常波动问题excel失败：{}" + e.getMessage() + e);
        }
    }

    @Override
    public void fusionAnalysisExcelOriginalOne(FusionQuery query, HttpServletResponse response) throws IOException, ParseException {
        List<OriginalData> list = new CopyOnWriteArrayList<>();
        Date startTime = DateUtils.parse(query.getStartTime(),"yyyy-MM-dd");
        String time = DateUtils.format(startTime,"yyyy-MM-dd");
        SearchRequest request = new SearchRequest(INDEX_SEAT_FK_REALTIMETRAJECTORY+time);
        // 构建查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("mergeTime").gte(query.getStartTime()).lte(query.getEndTime()));
        double[] doubles = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
        List<GeoPoint> points = new ArrayList<>();
        for(int i = 0 ;i < doubles.length;i=i+2) {
            points.add(new GeoPoint(doubles[i+1],doubles[i]));
        }
        boolQueryBuilder.filter(QueryBuilders.geoPolygonQuery("location", points));
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder
                .trackTotalHits(true)
                .sort(SortBuilders.fieldSort("mergeTime").order(SortOrder.ASC))
                .size(query.getPageSize())
                .timeout(TimeValue.timeValueHours(1L))
                .timeout(TimeValue.timeValueMinutes(30L))
                .timeout(TimeValue.timeValueSeconds(500L));
        request.source(searchSourceBuilder);
        SearchResponse search = client.search(request, COMMON_OPTIONS);
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        List<Map<String, Object>> inList = new CopyOnWriteArrayList<Map<String, Object>>();
        List<Map<String, Object>> outList = new CopyOnWriteArrayList<Map<String, Object>>();
        for (SearchHit hit : hits) {
            Map<String, Object> map = new ConcurrentHashMap();
            map = hit.getSourceAsMap();
            inList.add(map);
        }
        outList = change(inList, "mergeTarget", outList);
        outList.forEach(map->{
            JSONObject maps = JSONObject.parseObject(JSON.toJSONString(map));
            JSONArray jsonArray =JSONArray.parseArray(maps.getString("array"));
            if (jsonArray.size()>0) {
                OriginalData data = new OriginalData();
                JSONObject sourceItem = jsonArray.getJSONObject(0);
                data.setMergeTarget(sourceItem.getString("mergeTarget"));
                data.setNum(jsonArray.size());
                list.add(data);
            }
        });
        try {
            System.out.println("导出融合原始数据统计");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("融合原始数据统计", "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), OriginalData.class).sheet("融合原始数据统计")
                    .doWrite(list);
        } catch (Exception e) {
            // 重置response
            System.out.println("导出融合原始数据统计excel失败：{}" + e.getMessage() + e);
        }
    }

    @Override
    public void fusionAnalysisExcelOriginalTwo(FusionQuery query, HttpServletResponse response) throws IOException, ParseException {
        List<OriginalData> list = new CopyOnWriteArrayList<>();
        Date startTime = DateUtils.parse(query.getStartTime(),"yyyy-MM-dd");
        String time = DateUtils.format(startTime,"yyyy-MM-dd");
        SearchRequest request = new SearchRequest(INDEX_SEAT_TRACK_BEIDOU+time);
        // 构建查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("locationTime").gte(query.getStartTime()).lte(query.getEndTime()));
        double[] doubles = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
        List<GeoPoint> points = new ArrayList<>();
        for(int i = 0 ;i < doubles.length;i=i+2) {
            points.add(new GeoPoint(doubles[i+1],doubles[i]));
        }
        boolQueryBuilder.filter(QueryBuilders.geoPolygonQuery("location", points));
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder
                .trackTotalHits(true)
                .size(query.getPageSize())
                .sort(SortBuilders.fieldSort("locationTime").order(SortOrder.ASC))
                .timeout(TimeValue.timeValueHours(1L))
                .timeout(TimeValue.timeValueMinutes(30L))
                .timeout(TimeValue.timeValueSeconds(500L));
        request.source(searchSourceBuilder);
        SearchResponse search = client.search(request, COMMON_OPTIONS);
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        List<Map<String, Object>> inList = new CopyOnWriteArrayList<Map<String, Object>>();
        List<Map<String, Object>> outList = new CopyOnWriteArrayList<Map<String, Object>>();
        for (SearchHit hit : hits) {
            Map<String, Object> map = new ConcurrentHashMap();
            map = hit.getSourceAsMap();
            inList.add(map);
        }
        outList = change(inList, "devideNo", outList);
        outList.forEach(map-> {
            JSONObject maps = JSONObject.parseObject(JSON.toJSONString(map));
            JSONArray jsonArray = JSONArray.parseArray(maps.getString("array"));
            if (jsonArray.size() > 0) {
                OriginalData data = new OriginalData();
                JSONObject sourceItem = jsonArray.getJSONObject(0);
                data.setMergeTarget(sourceItem.getString("devideNo"));
                data.setNum(jsonArray.size());
                list.add(data);
            }
        });
        try {
            System.out.println("导出北斗原始数据统计");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("北斗原始数据统计", "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), OriginalData.class).sheet("北斗原始数据统计")
                    .doWrite(list);
        } catch (Exception e) {
            // 重置response
            System.out.println("导出北斗原始数据统计excel失败：{}" + e.getMessage() + e);
        }
    }

    @Override
    public void fusionAnalysisExcelOriginalThree(FusionQuery query, HttpServletResponse response) throws IOException, ParseException {
        List<OriginalData> list = new CopyOnWriteArrayList<>();
        Date startTime = DateUtils.parse(query.getStartTime(),"yyyy-MM-dd");
        String time = DateUtils.format(startTime,"yyyy-MM-dd");
        SearchRequest request = new SearchRequest(INDEX_SEAT_TRACK_RADAR+time);
        // 构建查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("receiveTime").gte(query.getStartTime()).lte(query.getEndTime()));
        double[] doubles = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
        List<GeoPoint> points = new ArrayList<>();
        for(int i = 0 ;i < doubles.length;i=i+2) {
            points.add(new GeoPoint(doubles[i+1],doubles[i]));
        }
        boolQueryBuilder.filter(QueryBuilders.geoPolygonQuery("location", points));
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder
                .trackTotalHits(true)
                .size(query.getPageSize())
                .sort(SortBuilders.fieldSort("receiveTime").order(SortOrder.ASC))
                .timeout(TimeValue.timeValueHours(1L))
                .timeout(TimeValue.timeValueMinutes(30L))
                .timeout(TimeValue.timeValueSeconds(500L));
        request.source(searchSourceBuilder);
        SearchResponse search = client.search(request, COMMON_OPTIONS);
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        List<Map<String, Object>> inList = new CopyOnWriteArrayList<Map<String, Object>>();
        List<Map<String, Object>> outList = new CopyOnWriteArrayList<Map<String, Object>>();
        for (SearchHit hit : hits) {
            Map<String, Object> map = new ConcurrentHashMap();
            map = hit.getSourceAsMap();
            inList.add(map);
        }
        outList = change(inList, "fusionBatchNum", outList);
        outList.forEach(map-> {
            JSONObject maps = JSONObject.parseObject(JSON.toJSONString(map));
            JSONArray jsonArray = JSONArray.parseArray(maps.getString("array"));
            if (jsonArray.size() > 0) {
                OriginalData data = new OriginalData();
                JSONObject sourceItem = jsonArray.getJSONObject(0);
                data.setMergeTarget(sourceItem.getString("fusionBatchNum"));
                data.setNum(jsonArray.size());
                list.add(data);
            }
        });
        try {
            System.out.println("导出天奥原始数据统计");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("天奥原始数据统计", "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), OriginalData.class).sheet("天奥原始数据统计")
                    .doWrite(list);
        } catch (Exception e) {
            // 重置response
            System.out.println("导出天奥原始数据统计excel失败：{}" + e.getMessage() + e);
        }
    }

    @Override
    public void fusionAnalysisExcelOriginalFour(FusionQuery query, HttpServletResponse response) throws IOException, ParseException {
        List<FuseOriginal> list = new CopyOnWriteArrayList<>();
        Date startTime = DateUtils.parse(query.getStartTime(),"yyyy-MM-dd");
        String time = DateUtils.format(startTime,"yyyy-MM-dd");
        SearchRequest request = new SearchRequest(INDEX_SEAT_REALTIMETRAJECTORY+time);
        // 构建查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("mergeTime").gte(query.getStartTime()).lte(query.getEndTime()));
        double[] doubles = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
        List<GeoPoint> points = new ArrayList<>();
        for(int i = 0 ;i < doubles.length;i=i+2) {
            points.add(new GeoPoint(doubles[i+1],doubles[i]));
        }
        boolQueryBuilder.filter(QueryBuilders.geoPolygonQuery("location", points));
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder
                .trackTotalHits(true)
                .sort(SortBuilders.fieldSort("mergeTime").order(SortOrder.ASC))
                .size(query.getPageSize())
                .timeout(TimeValue.timeValueHours(1L))
                .timeout(TimeValue.timeValueMinutes(30L))
                .timeout(TimeValue.timeValueSeconds(500L));
        request.source(searchSourceBuilder);
        SearchResponse search = client.search(request, COMMON_OPTIONS);
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        for (SearchHit hit : hits) {
            FuseOriginal fuseOriginal = JSONObject.parseObject(hit.getSourceAsString(), FuseOriginal.class);
            list.add(fuseOriginal);
        }
        try {
            System.out.println("导出融合原始数据");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("融合原始数据", "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), FuseOriginal.class).sheet("融合原始数据")
                    .doWrite(list);
        } catch (Exception e) {
            // 重置response
            System.out.println("导出融合原始数据excel失败：{}" + e.getMessage() + e);
        }
    }

    @Override
    public void fusionAnalysisExcelOriginalFive(FusionQuery query, HttpServletResponse response) throws IOException, ParseException {
        List<BeiDouOriginal> list = new CopyOnWriteArrayList<>();
        Date startTime = DateUtils.parse(query.getStartTime(),"yyyy-MM-dd");
        String time = DateUtils.format(startTime,"yyyy-MM-dd");
        SearchRequest request = new SearchRequest(INDEX_SEAT_TRACK_BEIDOU+time);
        // 构建查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("locationTime").gte(query.getStartTime()).lte(query.getEndTime()));
        //double[] doubles = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
        double[] doubles = LocationConstants.YANGPU;
        List<GeoPoint> points = new ArrayList<>();
        for(int i = 0 ;i < doubles.length;i=i+2) {
            points.add(new GeoPoint(doubles[i+1],doubles[i]));
        }
        boolQueryBuilder.filter(QueryBuilders.geoPolygonQuery("location", points));
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder
                .trackTotalHits(true)
                .size(query.getPageSize())
                .sort(SortBuilders.fieldSort("locationTime").order(SortOrder.ASC))
                .timeout(TimeValue.timeValueHours(1L))
                .timeout(TimeValue.timeValueMinutes(30L))
                .timeout(TimeValue.timeValueSeconds(500L));
        request.source(searchSourceBuilder);
        SearchResponse search = client.search(request, COMMON_OPTIONS);
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        for (SearchHit hit : hits) {
            BeiDouOriginal beiDouOriginal = JSONObject.parseObject(hit.getSourceAsString(), BeiDouOriginal.class);
            list.add(beiDouOriginal);
        }
        try {
            System.out.println("导出北斗原始数据");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("北斗原始数据", "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), BeiDouOriginal.class).sheet("北斗原始数据")
                    .doWrite(list);
        } catch (Exception e) {
            // 重置response
            System.out.println("导出北斗原始数据excel失败：{}" + e.getMessage() + e);
        }
    }

    @Override
    public void fusionAnalysisExcelOriginalTianao(FusionQuery query, HttpServletResponse response) throws IOException, ParseException {
        List<TianAoOriginal> list = new CopyOnWriteArrayList<>();
        Date startTime = DateUtils.parse(query.getStartTime(),"yyyy-MM-dd");
        String time = DateUtils.format(startTime,"yyyy-MM-dd");
        SearchRequest request = new SearchRequest(INDEX_SEAT_TRACK_RADAR+time);
        // 构建查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("receiveTime").gte(query.getStartTime()).lte(query.getEndTime()));
        double[] doubles = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
        List<GeoPoint> points = new ArrayList<>();
        for(int i = 0 ;i < doubles.length;i=i+2) {
            points.add(new GeoPoint(doubles[i+1],doubles[i]));
        }
        boolQueryBuilder.filter(QueryBuilders.geoPolygonQuery("location", points));
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder
                .trackTotalHits(true)
                .size(query.getPageSize())
                .sort(SortBuilders.fieldSort("receiveTime").order(SortOrder.ASC))
                .timeout(TimeValue.timeValueHours(1L))
                .timeout(TimeValue.timeValueMinutes(30L))
                .timeout(TimeValue.timeValueSeconds(500L));
        request.source(searchSourceBuilder);
        SearchResponse search = client.search(request, COMMON_OPTIONS);
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        for (SearchHit hit : hits) {
            TianAoOriginal tianAoOriginal = JSONObject.parseObject(hit.getSourceAsString(), TianAoOriginal.class);
            list.add(tianAoOriginal);
        }
        try {
            System.out.println("导出天奥原始数据");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("天奥原始数据", "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), TianAoOriginal.class).sheet("天奥原始数据")
                    .doWrite(list);
        } catch (Exception e) {
            // 重置response
            System.out.println("导出天奥原始数据excel失败：{}" + e.getMessage() + e);
        }
    }

    @Override
    public void fusionAnalysisExcelOriginalFk(FusionQuery query, HttpServletResponse response) throws IOException, ParseException {
        List<FuseOriginal> list = new CopyOnWriteArrayList<>();
        Date startTime = DateUtils.parse(query.getStartTime(),"yyyy-MM-dd");
        String time = DateUtils.format(startTime,"yyyy-MM-dd");
        SearchRequest request = new SearchRequest(INDEX_SEAT_FK_REALTIMETRAJECTORY+time);
        // 构建查询条件
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        boolQueryBuilder.filter(QueryBuilders.rangeQuery("mergeTime").gte(query.getStartTime()).lte(query.getEndTime()));
        double[] doubles = new double[]{109.05069874931793,19.75955316191097,109.20963772383247,19.811999020000147,109.35696654655628,19.782373115430687,109.33374624297481,19.66386949715284,109.09473725611036,19.617428889989903,109.05069874931793,19.75955316191097};
        List<GeoPoint> points = new ArrayList<>();
        for(int i = 0 ;i < doubles.length;i=i+2) {
            points.add(new GeoPoint(doubles[i+1],doubles[i]));
        }
        boolQueryBuilder.filter(QueryBuilders.geoPolygonQuery("location", points));
        searchSourceBuilder.query(boolQueryBuilder);
        searchSourceBuilder
                .trackTotalHits(true)
                .sort(SortBuilders.fieldSort("mergeTime").order(SortOrder.ASC))
                .size(query.getPageSize())
                .timeout(TimeValue.timeValueHours(1L))
                .timeout(TimeValue.timeValueMinutes(30L))
                .timeout(TimeValue.timeValueSeconds(500L));
        request.source(searchSourceBuilder);
        SearchResponse search = client.search(request, COMMON_OPTIONS);
        SearchHits searchHits = search.getHits();
        SearchHit[] hits = searchHits.getHits();
        for (SearchHit hit : hits) {
            FuseOriginal fuseOriginal = JSONObject.parseObject(hit.getSourceAsString(), FuseOriginal.class);
            list.add(fuseOriginal);
        }
        try {
            System.out.println("导出俯瞰融合原始数据");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode("俯瞰融合原始数据", "UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), FuseOriginal.class).sheet("俯瞰融合原始数据")
                    .doWrite(list);
        } catch (Exception e) {
            // 重置response
            System.out.println("导出俯瞰融合原始数据excel失败：{}" + e.getMessage() + e);
        }
    }

    /**
     * 按照List<Map<String,Object>>里面map的某个value重新封装成多个不同的list, 原始数据类型List<Map<String,Object>>, 转换后数据类型Map<String,List<Map<String,Object>>>
     * @param inList
     * @param oneMapKey
     * @param outList
     * @return
     */
    private static List<Map<String, Object>> change(List<Map<String, Object>> inList, String oneMapKey,
                                                    List<Map<String, Object>> outList) {
        // 1.将某个key的值存在set中
        Set<Object> setTmp = new CopyOnWriteArraySet<Object>();
        for (Map<String, Object> tmp : inList) {
            setTmp.add(tmp.get(oneMapKey));
        }
        // 2.遍历set
        Iterator<Object> it = setTmp.iterator();
        while (it.hasNext()) {
            String oneMapValueStr = "";
            String oneSetTmpStr = (String) it.next();
            Map<String, Object> oneSetTmpMap = new ConcurrentHashMap<String, Object>();
            List<Map<String, Object>> oneSetTmpList = new CopyOnWriteArrayList<Map<String, Object>>();

            for (Map<String, Object> tmp : inList) {
                oneMapValueStr = (String) tmp.get(oneMapKey);
                if (oneSetTmpStr.equals(oneMapValueStr)) {
                    oneSetTmpMap.put("text", oneSetTmpStr);
                    oneSetTmpList.add(tmp);
                }
            }
            oneSetTmpMap.put("array", oneSetTmpList);
            outList.add(oneSetTmpMap);
        }
        return outList;
    }


    public Long dateUtils(Long oneTime,Long twoTime){
        if (oneTime>twoTime){
            return oneTime-twoTime;
        }else {
            return twoTime-oneTime;
        }
    }

    public Double speedUtils(Double d,Double d2){
        if (d>d2){
            return d-d2;
        }else {
            return d2-d;
        }
    }
}